﻿using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using HeuristicLab.Common;
using HeuristicLab.MainForm;

namespace HeuristicLab.Problems.DataAnalysis.Views {
  [View("Target Response Gradient View")]
  [Content(typeof(IRegressionSolution))]
  public partial class RegressionSolutionTargetResponseGradientView : DataAnalysisSolutionEvaluationView {
    private ModifiableDataset internalDataset;
    private Dictionary<string, GradientChart> charts;
    private Dictionary<string, DoubleLimit> variableLimits;

    private const int Points = 1000;

    public RegressionSolutionTargetResponseGradientView() {
      InitializeComponent();
    }

    public new IRegressionSolution Content {
      get { return (IRegressionSolution)base.Content; }
      set {
        if (value == null || value == Content) return;
        base.Content = value;
      }
    }

    protected override void RegisterContentEvents() {
      base.RegisterContentEvents();
      variableListView.ItemChecked += variableListView_ItemChecked;
    }

    protected override void DeregisterContentEvents() {
      variableListView.ItemChecked -= variableListView_ItemChecked;
      base.DeregisterContentEvents();
    }

    protected override void OnContentChanged() {
      base.OnContentChanged();
      var pd = Content.ProblemData;
      var ds = pd.Dataset;

      // update variable list
      variableListView.Items.Clear();
      variableListView.Items.AddRange(pd.AllowedInputVariables.Select(x => new ListViewItem(x, 0)).ToArray());

      // update internal dataset
      var variableNames = pd.AllowedInputVariables.ToList();
      var variableValues = variableNames.Select(x => new List<double> { pd.Dataset.GetDoubleValues(x, pd.TrainingIndices).Median() });
      internalDataset = new ModifiableDataset(variableNames, variableValues);

      // update charts and variable limits
      charts = new Dictionary<string, GradientChart>();
      variableLimits = new Dictionary<string, DoubleLimit>();
      foreach (var x in pd.AllowedInputVariables) {
        double min = 0, max = 0;
        foreach (var v in ds.GetDoubleValues(x, pd.TrainingIndices)) {
          if (v > max) max = v;
          if (v < min) min = v;
        }
        variableLimits[x] = new DoubleLimit(min, max);
        var gradientChart = new GradientChart();
        gradientChart.Configure(new[] { Content }, pd, internalDataset, x, min, max, Points);
        charts[x] = gradientChart;
      }
    }

    private void variableListView_ItemChecked(object sender, ItemCheckedEventArgs e) {
      if (charts == null) return;
      var item = e.Item;
      var variable = item.Text;
      var chart = charts[variable];
      if (item.Checked) {
        gradientChartTableLayout.Controls.Add(chart);
        chart.UpdateChart();
      } else {
        gradientChartTableLayout.Controls.Remove(chart);
      }
    }
  }
}
